import AST_RENDERER from "./modules/astRenderer.js";
import ROUTE from "./modules/route";
import SCREEN from "./modules/screen";
import ROUTER from "./modules/router";

window.ROUTE = ROUTE;
window.SCREEN = SCREEN;
window.AST_RENDERER = AST_RENDERER;

export default class CUI_APP {

    constructor(sScriptSelector, aoGlobalTemplates, oGlobalDataContext) {

        this.sOrigin = window.location.origin;
        this.sLaunchPath = window.location.href;

        this.aoGlobalTemplates = (aoGlobalTemplates) ? aoGlobalTemplates : null;
        this.oGlobalDataContext = (oGlobalDataContext) ? oGlobalDataContext : null;

        this.dAppBody = null;

        if (sScriptSelector && typeof sScriptSelector === "string") {

            this.dAppScript = document.querySelector(sScriptSelector);
    
            if (this.dAppScript) {
    
                // Build the script path 
                this.sScriptPath = this.dAppScript.getAttribute('src').slice(0, this.dAppScript.getAttribute('src').lastIndexOf("/"));
            }
            else {
    
                console.error(`Unable find root script element by provided selector`);
            }
        }
        else {

            console.error(`SPA need to specify a root script used to define the overall project script root. Provided argument blank or not string`);
        }

        // Specify the router class, but let the owner define it.
        this.cRouter = null;

        // Default the application body to be the actual html body
        this.dAppBody = document.body;

        this.__bInit = false;
    }

    async _init(oRoutes) {

        console.log("Super app init!");

        let oDefinedRoutes = null;

        if (oRoutes) {

            oDefinedRoutes = oRoutes;
        }
        else if (window.O_SCREEN_ROUTES) {

            oDefinedRoutes = window.O_SCREEN_ROUTES;
        }

        // Define the application router
        this.cRouter = new ROUTER(O_SCREEN_ROUTES, this.sScriptPath);

        // Clear the body of the web application as a default ?

    }

    // ==================================
    // Common App Methods
    // ==================================

    // Common method to change out the application body
    setAppBody(dNode) {

        if (dNode) {

            this.dAppBody = dNode;
        }
    }

    newRendererInstance(oLocalTemplates, oLocalData) {

        return new AST_RENDERER(this.aoGlobalTemplates, this.oGlobalDataContext, oLocalTemplates, oLocalData);

    }

    // Declare the common method use
    navTo(sDestination, oNavData, eEvent) {

        if (this.cRouter && this.cRouter.navTo) {

            this.cRouter.navTo(sDestination, oNavData, eEvent, window.location.href);
        }
        else {

            console.error(`Provided router does not provide the required "navTo" function`)
        }
    }

}