import SIGNAL_MANAGER from './signalManager';

export default class DATA_STORE {
    constructor(params) {
        this.cSignals = new SIGNAL_MANAGER();
        this.state = (params && params.state) ? params.state : {};
        this.oNamedSignals = {
            stateChange:"stateChange"            
        }
    }

    getState(){
        return this.state;
    }

    setState(oNewState){
        if(oNewState){
            //Validate newState?

            this.state = oNewState;
        
            this.cSignals.emit(this.oNamedSignals.stateChange, self.state);
            
            return this.state;            
        }
        
        return null;
    }

    setValue(sDataKey, sNewValue){
        //Need to add minor validation here. 

        if(sNewValue){
            this.state[sDataKey] = sNewValue;

            this.cSignals.emit(this.oNamedSignals.stateChange, self.state);

            return this.state[sDataKey];    
        }

        return null;        
    }

    // Accepts a string data key or an array of nested dataKeys. 
    // TODO: Test and may need to update to parse datakey if '.' present in string data key
    getValue(dataKey){
       if(typeof dataKey == "string"){
            if(this.state.hasOwnProperty(dataKey)){
                return this.state[dataKey];    
            }
        }
        else if(Array.isArray(dataKey)){
            return this._checkNestedValue(this.state, dataKey);
        }       

        return null;        
    }

    _checkNestedValue(currentObj, dataKey){
        let nestedValue = null;
        let nestedDataKey = [...dataKey];//Shalow copy since we don't want to destruct original key array. 
        
        if(nestedDataKey.length > 1){
            let currentKey = nestedDataKey.shift();
            if(currentObj.hasOwnProperty(currentKey)){
                if(nestedDataKey.length>0){
                    nestedValue = this._checkNestedValue(currentObj[currentKey], nestedDataKey);
                }    
                else{
                    return currentObj[currentKey];
                }
            }
            else{
                return null;            
            }                       
        }
        else{
            if(currentObj.hasOwnProperty(nestedDataKey)){
                return currentObj[nestedDataKey];    
            }
            else{
                return null;
            }
        }

        return nestedValue;
    }

    //Expost a method of datastore so you don't need to access the store's event manager directly.
    //May need to add an event type propety, named event or specific dataKey mutation. 
    subscribe(sEventName, fCallback){
        return this.cSignals.subscribe(sEventName, fCallback);
    }

}