import DATA_INTERFACE from "./dataInterface";
import DATA_STORE from "./dataStore";

export default class ROUTE {

    constructor(oProps) {

        this.sTitle = null;

        this.oParentRoute = (oProps.oParentRoute) ? oProps.oParentRoute : null;
        this.oNavState = (oProps.oNavState) ? oProps.oNavState : null;

        this.dScreenContainer = (this.oParentRoute && this.oParentRoute.screenContainer) ? this.oParentRoute.screenContainer : cAPP.appBody; 

        this.cDataInterface = new DATA_INTERFACE();
        this.cDataInterface.init();

        this.oLifeCycleNames = {
            init: 'init',
            preRender:'preRender',
            render:'render',
            postRender:'postRender',
            unload:'unload',            
        }

        this._sGuardCycle = this.oLifeCycleNames.init;

        this.cDataStore = new DATA_STORE();        

        //Verify this being created here. 
        this.adScreenRegions = [];
    }

    get screenContainer(){
        return this.dScreenContainer;
    }

    set sGuardCycle(sCycle){
        if(sCycle && this.oLifeCycleNames.hasOwnProperty(sCycle)){
            this._sGuardCycle = sCycle;
        }
    }

    _guard(){
        if(this.guard){
            let guardResponse = this.guard();   
            
            if(guardResponse === true){
                
                 /*DEBUG: START*/
                console.log('Guard tripped - ABORT');
                // debugger;
                /*DEBUG: END*/
            }
            else if(typeof guardResponse == "string"){    // if string - nav there
                cAPP.navTo(guardResponse);                
            }
        }     
    }

    _checkGuard(sCycle){
        if(sCycle == this._sGuardCycle){
            this._guard();
        }
    }

    //================
    //Lifecycle methods
    //================

    _init () {
        // Call child method and next lifecycle method
        this._callChildMethod(this.oLifeCycleNames.init).then(()=>{
            this._checkGuard(this.oLifeCycleNames.init);
            this._preRender();
        })  
    
    }

    _preRender(){        
        // Call child method and next lifecycle method
        this._callChildMethod(this.oLifeCycleNames.preRender).then(()=>{
            this._checkGuard(this.oLifeCycleNames.preRender);
            this._render();
        })  
      
    }

    _render () {  
        // Call child method and next lifecycle method
        this._callChildMethod(this.oLifeCycleNames.render).then(()=>{
            this._checkGuard(this.oLifeCycleNames.render);
            this._postRender();
        })              
    }

    _postRender(){
        // Call child method and next lifecycle method
        this._callChildMethod(this.oLifeCycleNames.postRender).then(()=>{
            this._checkGuard(this.oLifeCycleNames.postRender);            
        })  
    }

    _unload(){
        /*DEBUG: START*/
        console.log('ROUTE - Unload - '+ this.sTitle);
        /*DEBUG: END*/
        this._callChildMethod('unload');
    }

    async _callChildMethod(sMethod){
        if(this[sMethod] && typeof this[sMethod] == "function"){
            this[sMethod]();
        }
    }

     _registerService(sServiceName, oServiceProps){
        if(sServiceName && oServiceProps){

            if(!this.cDataInterface){
                this.cDataInterface = new DATA_INTERFACE();
                this.cDataInterface.init();
            }

            return this.cDataInterface.registerService(sServiceName, oServiceProps);
        }
        return null;
    }
 
    _registerDataStore(sDataStoreName, oDataStoreProps){
        if(sDataStoreName && oDataStoreProps){
            
            if(!this.cDataInterface){
                this.cDataInterface = new DATA_INTERFACE();
                this.cDataInterface.init();
            }

            return this.cDataInterface.registerDataStore(sDataStoreName, oDataStoreProps);
        }
        return null;
    }
    
}